﻿using Cop.Borovkov.Var3.Models;

namespace Cop.Borovkov.Var3.Components
{
    /// <summary>
    /// Визуальный компонент вывода таблицы значений
    /// </summary>
    public partial class CustomDataTable : UserControl
    {
        /// <summary>
        /// </summary>
        public CustomDataTable()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Определить структуру таблицы 
        /// </summary>
        /// <param name="columnParameters"></param>
        public void ConfigureColumns(params CustomDataTableColumnParameter[] columnParameters)
        {
            outDataGridView.Columns.Clear();

            foreach (var columnParametr in columnParameters)
            {
                DataGridViewTextBoxColumn column = new()
                {
                    Name = columnParametr.HeaderName,
                    HeaderText = columnParametr.HeaderName,
                    DataPropertyName = columnParametr.PropertyName,
                    Visible = columnParametr.Visible,
                };

                if (columnParametr.Width != 0)
                {
                    column.Width = columnParametr.Width;
                }
                else
                {
                    column.AutoSizeMode = DataGridViewAutoSizeColumnMode.DisplayedCells;
                }

                _ = outDataGridView.Columns.Add(column);
            }
        }

        /// <summary>
        /// Отчистить таблицу
        /// </summary>
        public void Clear()
            => outDataGridView.Rows.Clear();

        /// <summary>
        /// Индекс выбранной строки
        /// </summary>
        public int SelectedRow
        {
            get => outDataGridView.CurrentRow.Index;

            set
            {
                outDataGridView.CurrentRow.Selected = false;
                outDataGridView.Rows[value].Selected = true;
            }
        }

        /// <summary>
        /// Получить стоку таблицы в виде объекта
        /// </summary>
        /// <typeparam name="TType"></typeparam>
        /// <param name="rowIndex"></param>
        /// <returns></returns>
        public TType? GetRow<TType>(int rowIndex) where TType : new()
        {
            Type type = typeof(TType);
            var element = new TType();

            if (element is null)
            {
                return default;
            }

            var row = outDataGridView.Rows[rowIndex].Cells;

            for (int i = 0; i < row.Count; i++)
            {
                if (string.IsNullOrEmpty(row[i].OwningColumn.DataPropertyName))
                {
                    continue;
                }

                var field = type.GetProperty(row[i].OwningColumn.DataPropertyName);
                field?.SetValue(element, row[i].Value);
            }

            return element;
        }

        /// <summary>
        /// Заполнить таблицу
        /// </summary>
        /// <typeparam name="TType"></typeparam>
        /// <param name="insertValues"></param>
        public void Fill<TType>(IList<TType> insertValues)
        {
            Clear();

            Type type = typeof(TType);

            for (int i = 0; i < insertValues.Count; ++i)
            {
                var row = insertValues[i];
                _ = outDataGridView.Rows.Add();

                for (int j = 0; j < outDataGridView.ColumnCount; ++j)
                {
                    outDataGridView.Rows[i].Cells[j].Value 
                        = type.GetProperty(outDataGridView.Columns[j].DataPropertyName)?.GetValue(row)!;
                }
            }
        }
    }
}
